function [sys,E]=Morse(Ep0,x0,a,m,varargin)
% MORSE determina las energías de los estados ligados de un objeto físico que 
%  interactúa con una energía potencial de Morse.
%
%   [sys,E]=Morse(Ep0,x0,a,m,varargin) 
%					-- varargin indica varios argumentos de entrada
%   	>> Ep0          = Profundidad del pozo (eV)
%   	>> x0           = Posición del mínimo del pozo energético
%   	>> a            = Constante característica (m^{-1})
%   	>> m            = Masa del objeto físico (respecto a la del electrón)
%   	>> varargin     = Argumentos de entrada opcionales
%           >> nx     = Número de puntos en el eje Ox (defecto: 200)
%           >> nE     = Número de valores de energía (defecto: 1000)
%           >> Emin   = Valor inferior de la energía (defecto: min(Ep))
%           >> Emax   = Valor superior de la energía (defecto: 0)
%
% Olivier Pujol (2010)

%% Verificaciones

if (Ep0<=0),error('Se debe tener Ep0 > 0');end
if (x0<=0),error('Se debe tener x0 > 0');end
if (m<=0),error('Se debe tener m > 0');end

%% Argumentos opcionales de entrada

switch length(varargin)
    case 0
        nx=200;                     % Valor por defecto del número de puntos en Ox
        nE=1000;                    % Valor por defecto del número de valores de la energía
    case 1
        nx=varargin{1};             % Valor por defecto del número de puntos en Ox
        nE=1000;                    % Valor por defecto del número de valores de la energía
    case 2
        nx=varargin{1};             % Valor por defecto del número de puntos en Ox
        nE=varargin{2};             % Valor por defecto del número de valores de la energía
    case 4
        nx=varargin{1};             % Valor del número de puntos en Ox
        nE=varargin{2};             % Valor del número de valores de la energía
        E=linspace(varargin{3},varargin{4},nE);              % Vector de los valores de la energía
    otherwise
        ('¡Número incorrecto de argumentos opcionales de entrada!');
end

%% Determinación de los límites del pozo

% La energía potencial es teóricamente infinita en x=0.   
% Consecuentemente, el límite inferior del pozo se escoge así: Ep = n*Ep0 (n=10).
% El usuario puede cambiar este valor. 

n=10;
xA=x0-(1/a)*log(1+(n)^0.5);
clear n

% La energía potencial vale teóricamente cero cuando x es infinito. Consecuentemente, 
% el límite superior se escoge así: Ep = epsilon*Ep0 (epsilon=0.001).
% El usuario puede cambiar este valor. 

epsilon = 0.001;
xB=x0-(1/a)*log(1-(1-epsilon)^0.5);
clear epsilon

if (xA>=x0),error('El límite inferior del pozo es más grande que x0');end
if (xB<=x0),error('El límite superior del pozo es más pequeño que x0');end
if (xA>=xB),error('El límite inferior del pozo es más grande que su límite superior');end

%% Energía potencial

% Construcción
x=linspace(xA,xB,nx);                % Intervalo espacial 
Epc=Ep0*(1-exp(-a*(x-x0))).^2;       % Energía potencial de Morse (Morse 1929). El origen es el mínimo

% Digitalización
[M,xt,Ep]=digitalización(x,Epc); 
axis([0 xB 0 Ep0]);            		% Ejes del gráfico

% Valores de la energía
if (length(varargin)~=4)
    E=linspace(min(Ep),Ep0,nE);   % Vector de los valores de la energía
end    

if (min(E)<0 || max(E)>Ep0),error('Algunos valores de la energía no están en el dominio de los estados ligados');end

%% Estados ligados (o confinados)

[sys,E]=Epot(M,xt,Ep,m,E);          % Sistema 
sisconf(sys,E);                     % Estados ligados (o confinados)

return;