function [sys,E]=OscAnArm(L,Ep0,C,m,varargin)
% OscAnArm determina las energas de los estados ligados de un objeto fsico   
%  confinado en un pozo anarmnico (cbico) de energa potencial
%  
%  [sys,E]=OscAnArm(L,Ep0,C,m,varargin) 
%					-- varargin indica varios argumentos de entrada
%   	>> L    	 = Anchura del pozo (nm)
%   	>> Ep0  	 = Profundidad del pozo (eV) (Ep0 < 0)
%   	>> C    	 = Constante 
%   	>> m    	 = Masa del objeto fsico (respecto a la del electrn)
%   	>> varargin  = Argumentos de entrada opcionales
%           > nx   = Nmero de puntos en el eje Ox (defecto: 100)
%           > nE   = Nmero de valores de energa (defecto: 500)
%           > Emin = Valor inferior de la energa (defecto: min(Ep))
%           > Emax = Valor superior de la energa (defecto: 0)
%
% Olivier Pujol (2010)

%% Verificaciones

if (length(varargin)>4), error('Nmero incorrecto de argumentos opcionales de entrada!');end
if (nargin-length(varargin)~=4),error('Nmero incorrecto de argumentos no opcionales de entrada!');end

%% Verificacin del valor de C

if (C==0),error('C=0! Es un oscilador armnico');end

Clim=16*abs(Ep0)/(3*sqrt(3)*L^3);
if (abs(C)>Clim),error('El valor de C es demasiado grande');end        

clear Clim

%% Determinacin de los lmites del pozo

polinomio=[-C 4*Ep0/L^2 0 -Ep0];     % Tres soluciones reales si abs(C)<Clim
racine=roots(polinomio);             % Soluciones del polinomio

if (isreal(racine)==0)              % Son reales las soluciones?
    error('Hay soluciones complejas (el valor de C es demasiado grande)');
end

% Los lmites del pozo son las dos soluciones ms pequeas. La solucin ms grande no tiene inters.

        if (C>0)
            xA=racine(3);    % Lmite inferior
            xB=racine(2);    % Lmite superior
        elseif (C<0)
            xA=racine(2);    % Lmite inferior
            xB=racine(3);    % Lmite superior
        end

if (xA>xB)
    error('Intervalo espacial incorrecto: el lmite inferior es ms grande que el lmite superior');
end

clear polinomio racine

%% Argumentos opcionales de entrada

switch length(varargin)
    case 0
        nx=100;                     % Valor por defecto del nmero de puntos en Ox
        nE=500;                     % Valor por defecto del nmero de valores de la energa
    case 1
        nx=varargin{1};             % Valor por defecto del nmero de puntos en Ox
        nE=500;                     % Valor por defecto del nmero de valores de la energa
    case 2
        nx=varargin{1};             % Valor por defecto del nmero de puntos en Ox
        nE=varargin{2};             % Valor por defecto del nmero de valores de la energa
    case 4
        nx=varargin{1};             % Valor del nmero de puntos en Ox 
        nE=varargin{2};             % Valor del nmero de valores de la energa
        E=linspace(varargin{3},varargin{4},nE);     % Vector de los valores de la energa
    otherwise
        ('Nmero incorrecto de argumentos opcionales de entrada!');
end

%% Energa potencial

% Construccin
x=linspace(xA,xB,nx);             % Intervalo espacial
clear xA xB
Epc=4*Ep0*(x/L).^2-C*x.^3;        % Energa potencial (perturbacin cbica anarmnica)

% Digitalizacin
[M,xt,Ep]=digitalizacin(x,Epc);

% Valores de la energa
if (length(varargin)~=4)
    E=linspace(min(Ep),Ep0,nE);       % Vector de los valores de la energa
end    

if (max(E)>Ep0)
    error('Algunos valores de la energa no estn en el dominio de los estados ligados');
end

%% Estados ligados (o confinados)

[sys,E]=Epot(M,xt,Ep,m,E);          % Sistema 
sisconf(sys,E);                     % Estados ligados (o confinados)

return;