function [x,psi,rho]=psirho(sys,incremento,xmin,xmax,TES,E)
% PSIRHO determina la función de onda y la densidad lineal de probabilidad de  
%   un objeto físico que interactúa con una energía potencial. 
%
%   [x,psi,rho]=PSIRHO(sys,pas,xmin,xmax)
%       >> sys 			= Sistema
%       >> incremento 	= Incremento en el eje Ox
%       >> xmin 		= Límite inferior de x en el eje Ox
%       >> xmax 		= Límite superior de x en el eje Ox 
%       << x 			= Posición en el eje Ox
%       << psi 			= Amplitud compleja de la función de onda del objeto físico  
%       << rho 			= Densidad lineal de probabilidad del objeto físico
%
%   Olivier Pujol (2010)
%
%% Verificaciones

if (incremento>=0.01),error('¡El incremento es demasiado grande!');end
if ( (xmin>sys{1}{1}(5))||(xmax<sys{1}{1}(6)) ),error('¡El sistema está fuera del intervalo espacial!');end

%% Estado de difusión o estado ligado (confinado)

iconf=input('¿Se trata de un estado ligado? sí (escriba 1) / no (escriba 0) ');

switch class(sys)
    case 'escalón' 
        [x,rho]=psirho(sys,incremento,xmin,xmax);
    case 'cell'
        
%% Se debe entrar el valor de En. 
%% Se considera el índice iE que corresponde a la diferencia mínima entre En y E

if (length(E)>=2)
    En=input('Elija un valor de En: ');
    Ep=nan(1,length(sys{1}));
    Ep(1)=sys{1}{1}(1);
    for ic=2:length(sys{1})
        Ep(ic)=get(sys{1}{ic},'EnergíaPotencialSalida');
    end

    iE=find(abs(abs(En-E)-min(abs(En-E)))<eps,1,'first');
end
if (length(E)==1),iE=1;end

%% Inicialización

x=cell(1,length(sys{iE}));      % Intervalo espacial
psi=cell(1,length(sys{iE}));    % Función de onda
rho=cell(1,length(sys{iE}));    % Densidad lineal de probabilidad
    
%% Estado de difusión

if (iconf==0)
        
        % Cálculo en la región de entrada
        x{1}=linspace(xmin,sys{iE}{1}(5),1+(sys{iE}{1}(5)-xmin)/incremento);
        k=sys{iE}{1}(3);
        Amp=[1;-TES{iE}(2,1)/TES{iE}(2,2)];
        psi{1}=nan(size(x{1}));
        psi{1}=(1/sqrt(k))*( Amp(1)*exp(i*k*(x{1}-sys{iE}{1}(5))) ...
                            +Amp(2)*exp(-i*k*(x{1}-sys{iE}{1}(5))) );
        rho{1}=cell(size(x{1}));
        rho{1}=psi{1}.*conj(psi{1});
        
        % Cálculo en las otras regiones (excepto la región de salida)
        for n=2:length(sys{iE})-1
            Amp=matriz(sys{iE}{n})*Amp;
            x1=get(sys{iE}{n},'Posición');
            x2=get(sys{iE}{n+1},'Posición');
            x{n}=linspace(x1,x2,1+(x2-x1)/incremento);
            k=get(sys{iE}{n},'VectorOndaSalida');
            psi{n}=nan(size(x{n}));
            psi{n}=(1/sqrt(k))*( Amp(1)*exp(i*k*(x{n}-x1)) ...
                                +Amp(2)*exp(-i*k*(x{n}-x1)) );
            rho{n}=nan(size(x{n}));
            rho{n}=psi{n}.*conj(psi{n});
            Amp=translación(sys{iE}{n},sys{iE}{n+1})*Amp;
        end

        % Cálculo en la región de salida
        x{n+1}=linspace(sys{iE}{1}(6),xmax,(xmax-sys{iE}{1}(6))/incremento);
        k=sys{iE}{1}(4);
        psi{n+1}=nan(size(x{n+1}));
        psi{n+1}=(1/sqrt(k))*( (1/TES{iE}(2,2))*exp(i*k*(x{n+1}-sys{iE}{1}(6))) );
        rho{n+1}=nan(size(x{n+1}));
        rho{n+1}=psi{n+1}.*conj(psi{n+1});

end

%% Estado ligado (o confinado)

if (iconf==1)
        x{1}=linspace(xmin,sys{iE}{1}(5),1+(sys{iE}{1}(5)-xmin)/incremento);
        k=sys{iE}{1}(3);
        Amp=[0;1];
        psi{1}=nan(size(x{1}));
        psi{1}=(1/sqrt(k))*( Amp(2)*exp(-i*k*(x{1}-sys{iE}{1}(5))) );
        rho{1}=nan(size(x{1}));
        rho{1}=psi{1}.*conj(psi{1});
        
        % Cálculo en las otras regiones (excepto la región de salida)
        for n=2:length(sys{iE})-1
            Amp=matriz(sys{iE}{n})*Amp;
            x1=get(sys{iE}{n},'Posición');
            x2=get(sys{iE}{n+1},'Posición');
            x{n}=linspace(x1,x2,1+(x2-x1)/incremento);
            k=get(sys{iE}{n},'VectorOndaSalida');
            psi{n}=nan(size(x{n}));
            psi{n}=(1/sqrt(k))*( Amp(1)*exp(i*k*(x{n}-x1)) ...
                                +Amp(2)*exp(-i*k*(x{n}-x1)) );
            rho{n}=nan(size(x{n}));
            rho{n}=psi{n}.*conj(psi{n});
            Amp=translación(sys{iE}{n},sys{iE}{n+1})*Amp;
        end

        % Cálculo en la región de salida
        x{n+1}=linspace(sys{iE}{1}(6),xmax,(xmax-sys{iE}{1}(6))/incremento);
        k=sys{iE}{1}(4);
        Amp=matriz(sys{iE}{length(sys{iE})})*Amp;
        psi{n+1}=nan(size(x{n+1}));
        psi{n+1}=(1/sqrt(k))*( Amp(1)*exp(i*k*(x{n+1}-sys{iE}{1}(6))) );
        rho{n+1}=nan(size(x{n+1}));
        rho{n+1}=psi{n+1}.*conj(psi{n+1});    

end 

        otherwise
        error('Tipo de argumento incorrecto');

end

%% Normalización por la superficie (área) bajo la curva de rho(x) entre xmin y xmax.

área=0;
for iz=1:length(sys{iE})
área = área+sum((rho{iz}(1:end-1)+rho{iz}(2:end))*incremento)/2;
end
for iz=1:length(sys{iE})
    rho{iz}=rho{iz}/área;
    psi{iz}=psi{iz}/sqrt(área);
end

clear Amp área iz

return;