function [En,sysfin,Efin,TESfin]=refine(sys,m,En,varargin)
%   REFINE determina con ms precisin la energa de un estado ligado (confinado). 
%
%   [En,sysfin,Efin,TESfin]=REFINE(sys,m,En,varargin)
%					-- varargin indica varios argumentos de entrada
%       >> sys          = Sistema
%       >> m            = Masa del objeto fsico (respecto a la del electrn)
%       >> En           = Energa para determinar con precisin
%       >> varargin     = Argumentos de entrada opcionales
%            > alpha    = Factor (<< 1) para que 2*alpha*En sea el nuevo  
					intervalo energtico centrado en En
%            > nE       = Nmero de valores de energa
%            > Emin     = Valor inferior de la energa del nuevo intervalo
%            > Emax     = Valor superior de la energa del nuevo intervalo
%       << En           = Valor refinada de la energe del estado ligado
%       << sysfin       = Nuevo sistema relativo al nuevo intervalo
%       << Efin         = Valores de la energa en el nuevo intervalo
%       << TESfin       = Matriz de transferencia de sysfin
%
%   Olivier Pujol (2010)
%
%% Recogemos las caractersticas del sistema

xt=nan(1,length(sys{1})-1);
Ep=nan(1,length(sys{1}));
M=cell(1,length(sys{1}));

Ep(1)=sys{1}{1}(1);
for ic=2:length(sys{1})
    xt(ic-1)=get(sys{1}{ic},'Posicin');
    Ep(ic)=get(sys{1}{ic},'EnergaPotencialSalida');
    M{ic-1}=char(get(sys{1}{ic},'Nombre'));
end

clear sys

%% Argumentos opcionales de entrada

switch length(varargin)
    case 0
        alpha=0.01;                 % Valor por defecto del percentaje 
        nE=400;                     % Valor por defecto del nmero de valores de la energa
    case 1
        alpha=varargin{1};          % Valor por defecto del percentaje 
        nE=400;                     % Valor por defecto del nmero de valores de la energa
    case 2
        alpha=varargin{1};          % Valor por defecto del percentaje 
        nE=varargin{2};             % Valor por defecto del nmero de valores de la energa
    case 3
        Efin=linspace(varargin{1},varargin{2},varargin{3});     % Vector energa. El origen es el mnimo
    otherwise
        ('Nmero incorrecto de argumentos opcionales de entrada!');
end

%% Clculo de los valores refinados de En 

%   Alrededor del valor aproximado de En, se define un intervalo de +/- alpha*En
%   (alpha < 1; defecto: 0,01).

if (length(varargin)~=3)
    Emin=(1-alpha)*En;
    Emax=(1+alpha)*En;
    Efin=linspace(Emin,Emax,nE);
end

%   Se tiene que volver a calcular el sistema sobre el intervalo energtico nuevamente considerado

[sysfin,Efin]=Epot(M,xt,Ep,m,Efin);
TESfin=matriz(sysfin);
clear M xt En

dfin=nan(1,length(TESfin));         % Inicializacin
for iT=1:size(TESfin,2)
    dfin(iT)=TESfin{iT}(2,2);       % Elemento d
end

En=minimum(Efin,dfin);

%% Grfico

figure
plot(log(abs(dfin)),Efin,'b');
axis([min(log(abs(dfin))) max(log(abs(dfin))) min(Efin) max(Efin)]); 
set(gca,'xminortick','on','yminortick','on');
grid on;
box on
title('Zum de la energa del estado ligado');
xlabel('ln(|d|)');
ylabel('Energa (eV)','Color','k');

return;

function Econf=minimum(Efin,dfin)
%
derivee=diff(log(abs(dfin)));
Ind=derivee(1:end-1).*derivee(2:end) <=0;
Econf=Efin(Ind==1);

return;
